﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Dongle;

namespace Sample03
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        /************************************************************************/
        /*                              结构                                    */
        /************************************************************************/
        //RSA公钥格式(兼容1024,2048)
        [StructLayout(LayoutKind.Sequential)]
        public struct RSA_PUBLIC_KEY
        {
            public uint bits;                   // length in bits of modulus        	
            public uint modulus;				  // modulus
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public byte[] exponent;       // public exponent
        }
        //RSA私钥格式(兼容1024,2048)
        [StructLayout(LayoutKind.Sequential)]
        public struct RSA_PRIVATE_KEY
        {
            public uint bits;                   // length in bits of modulus        	
            public uint modulus;				  // modulus  
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public byte[] publicExponent;       // public exponent
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public byte[] exponent;       // public exponent
        }
        //外部ECCSM2公钥格式 ECC(支持bits为192或256)和SM2的(bits为固定值0x8100)公钥格式
        [StructLayout(LayoutKind.Sequential)]
        public struct ECCSM2_PUBLIC_KEY
        {
            public uint bits;                   // length in bits of modulus        	
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
            public uint[] XCoordinate;       // 曲线上点的X坐标
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
            public uint[] YCoordinate;       // 曲线上点的Y坐标
        }
        //外部ECCSM2私钥格式 ECC(支持bits为192或256)和SM2的(bits为固定值0x8100)私钥格式  
        [StructLayout(LayoutKind.Sequential)]
        public struct ECCSM2_PRIVATE_KEY
        {
            public uint bits;                   // length in bits of modulus        	
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
            public uint[] PrivateKey;           // 私钥
        }

        //加密锁信息
        [StructLayout(LayoutKind.Sequential)]
        public struct DONGLE_INFO
        {
            public ushort m_Ver;               //COS版本,比如:0x0201,表示2.01版             	
            public ushort m_Type;              //产品类型: 0xFF表示标准版, 0x00为时钟锁,0x01为带时钟的U盘锁,0x02为标准U盘锁  
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 8)]
            public string m_BirthDay;       //出厂日期 
            public uint m_Agent;             //代理商编号,比如:默认的0xFFFFFFFF
            public uint m_PID;               //产品ID
            public uint m_UserID;            //用户ID
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 8)]
            public string m_HID;            //8字节的硬件ID
            public uint m_IsMother;          //母锁标志: 0x01表示是母锁, 0x00表示不是母锁
            public uint m_DevType;           //设备类型(PROTOCOL_HID或者PROTOCOL_CCID)
        }
        /*************************文件授权结构***********************************/
        //数据文件授权结构
        [StructLayout(LayoutKind.Sequential)]
        public struct DATA_LIC
        {
            public ushort m_Read_Priv;     //读权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限            	
            public ushort m_Write_Priv;    //写权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限
        }

        //私钥文件授权结构
        [StructLayout(LayoutKind.Sequential)]
        public struct PRIKEY_LIC
        {
            public uint m_Count;        //可调次数: 0xFFFFFFFF表示不限制, 递减到0表示已不可调用
            public byte m_Priv;         //调用权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限
            public byte m_IsDecOnRAM;   //是否是在内存中递减: 1为在内存中递减，0为在FLASH中递减
            public byte m_IsReset;      //用户态调用后是否自动回到匿名态: TRUE为调后回到匿名态 (开发商态不受此限制)
            public byte m_Reserve;      //保留,用于4字节对齐
        }

        //对称加密算法(SM4/TDES)密钥文件授权结构
        [StructLayout(LayoutKind.Sequential)]
        public struct KEY_LIC
        {
            public uint m_Priv_Enc;   //加密时的调用权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限
        }


        //可执行文件授权结构
        [StructLayout(LayoutKind.Sequential)]
        public struct EXE_LIC
        {
            public ushort m_Priv_Exe;   //运行的权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限
        }

        /****************************文件属性结构********************************/
        //数据文件属性数据结构
        [StructLayout(LayoutKind.Sequential)]
        public struct DATA_FILE_ATTR
        {
            public uint m_Size;      //数据文件长度，该值最大为4096
            public DATA_LIC m_Lic;       //授权
        }

        //ECCSM2/RSA私钥文件属性数据结构
        [StructLayout(LayoutKind.Sequential)]
        public struct PRIKEY_FILE_ATTR
        {
            public ushort m_Type;       //数据类型:ECCSM2私钥 或 RSA私钥
            public ushort m_Size;       //数据长度:RSA该值为1024或2048, ECC该值为192或256, SM2该值为0x8100
            public PRIKEY_LIC m_Lic;        //授权
        }

        //对称加密算法(SM4/TDES)密钥文件属性数据结构
        [StructLayout(LayoutKind.Sequential)]
        public struct KEY_FILE_ATTR
        {
            public uint m_Size;       //密钥数据长度=16
            public KEY_LIC m_Lic;        //授权
        }

        //可执行文件属性数据结构
        [StructLayout(LayoutKind.Sequential)]
        public struct EXE_FILE_ATTR
        {
            public EXE_LIC m_Lic;        //授权	
            public ushort m_Len;        //文件长度
        }
        /*************************文件列表结构***********************************/
        //获取私钥文件列表时返回的数据结构
        [StructLayout(LayoutKind.Sequential)]
        public struct PRIKEY_FILE_LIST
        {
            public ushort m_FILEID;  //文件ID
            public ushort m_Reserve; //保留,用于4字节对齐
            public PRIKEY_FILE_ATTR m_attr;    //文件属性
        }

        //获取SM4及TDES密钥文件列表时返回的数据结构
        [StructLayout(LayoutKind.Sequential)]
        public struct KEY_FILE_LIST
        {
            public ushort m_FILEID;  //文件ID
            public ushort m_Reserve; //保留,用于4字节对齐
            public KEY_FILE_ATTR m_attr;    //文件属性
        }

        //获取数据文件列表时返回的数据结构
        [StructLayout(LayoutKind.Sequential)]
        public struct DATA_FILE_LIST
        {
            public ushort m_FILEID;  //文件ID
            public ushort m_Reserve; //保留,用于4字节对齐
            public DATA_FILE_ATTR m_attr;    //文件属性
        }

        //获取可执行文件列表时返回的数据结构
        [StructLayout(LayoutKind.Sequential)]
        public struct EXE_FILE_LIST
        {
            public ushort m_FILEID;    //文件ID
            public EXE_FILE_ATTR m_attr;
            public ushort m_Reserve;  //保留,用于4字节对齐
        }

        //下载和列可执行文件时填充的数据结构
        [StructLayout(LayoutKind.Sequential)]
        public struct EXE_FILE_INFO
        {
            public ushort m_dwSize;           //可执行文件大小
            public ushort m_wFileID;          //可执行文件ID
            public byte m_Priv;             //调用权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限

            public byte[] m_pData;            //可执行文件数据
        }


        //需要发给空锁的初始化数据
        [StructLayout(LayoutKind.Sequential)]
        public struct SON_DATA
        {
            public int m_SeedLen;                 //种子码长度
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public string m_SeedForPID;	       //产生产品ID和开发商密码的种子码 (最长250个字节)
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 18)]
            public string m_UserPIN;         //用户密码(16个字符的0终止字符串)
            public sbyte m_UserTryCount;            //用户密码允许的最大错误重试次数
            public sbyte m_AdminTryCount;           //开发商密码允许的最大错误重试次数
            //RSA_PRIVATE_KEY m_UpdatePriKey;   //远程升级私钥
            public int m_UserID_Start;            //起始用户ID
        }

        //母锁数据
        [StructLayout(LayoutKind.Sequential)]
        public struct MOTHER_DATA
        {
            public SON_DATA m_Son;                  //子锁初始化数据
            public int m_Count;                //可产生子锁初始化数据的次数 (-1表示不限制次数, 递减到0时会受限)
        }





        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_Enum(ref DONGLE_INFO pDongleInfo, out ushort pCount);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_Open(ref uint phDongle, int nIndex);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_Close(uint hDongle);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_VerifyPIN(uint hDongle, uint nFlags, byte[] pPIN, out int pRemainCount);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_CreateFile(uint hDongle, uint nFileType, ushort wFileID, uint pFileAttr);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_WriteFile(uint hDongle, uint nFileType, ushort wFileID, short wOffset, byte[] buffer, int nDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_ReadFile(uint hDongle, short wFileID, short wOffset, byte[] buffer, int nDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_ListFile(uint hDongle, uint nFileType, DATA_FILE_LIST[] pFileList, ref int pDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_DeleteFile(uint hDongle, uint nFileType, short wFileID);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_DownloadExeFile(uint hDongle, EXE_FILE_INFO[] pExeFileInfo, int nCount);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_RunExeFile(uint hDongle, short wFileID, byte[] pInOutData, short wInOutDataLen, ref int nMainRet);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_WriteShareMemory(uint hDongle, byte[] pData, int nDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_ReadShareMemory(uint hDongle, byte[] pData);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_WriteData(uint hDongle, int nOffset, byte[] pData, int nDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_ReadData(uint hDongle, int nOffset, byte[] pData, int nDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_LEDControl(uint hDongle, uint nFlag);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_SwitchProtocol(uint hDongle, uint nFlag);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_GetUTCTime(uint hDongle, ref uint pdwUTCTime);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_SetDeadline(uint hDongle, uint dwTime);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_GenUniqueKey(uint hDongle, int nSeedLen, byte[] pSeed, byte[] pPIDstr, byte[] pAdminPINstr);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_ResetState(uint hDongle);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_ChangePIN(uint hDongle, uint nFlags, byte[] pOldPIN, byte[] pNewPIN, int nTryCount);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_RFS(uint hDongle);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_SetUserID(uint hDongle, uint dwUserID);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_ResetUserPIN(uint hDongle, byte[] pAdminPIN);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_RsaGenPubPriKey(uint hDongle, ushort wPriFileID, ref RSA_PUBLIC_KEY pPubBakup, ref RSA_PRIVATE_KEY pPriBakup);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_RsaPri(uint hDongle, ushort wPriFileID, uint nFlag, byte[] pInData, uint nInDataLen, byte[] pOutData, ref uint pOutDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_RsaPub(uint hDongle, uint nFlag, ref RSA_PUBLIC_KEY pPubKey, byte[] pInData, uint nInDataLen, byte[] pOutData, ref uint pOutDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_TDES(uint hDongle, ushort wKeyFileID, uint nFlag, byte[] pInData, byte[] pOutData, uint nDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_SM4(uint hDongle, ushort wKeyFileID, uint nFlag, byte[] pInData, byte[] pOutData, uint nDataLen);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_DeleteFile(uint hDongle, uint nFileType, ushort wFileID);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_HASH(uint hDongle, uint nFlag, byte[] pInData, uint nDataLen, byte[] pHash);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_LimitSeedCount(uint hDongle, int nCount);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_Seed(uint hDongle, byte[] pSeed, uint nSeedLen, byte[] pOutData);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_EccGenPubPriKey(uint hDongle, ushort wPriFileID, ref ECCSM2_PUBLIC_KEY vPubBakup, ref ECCSM2_PRIVATE_KEY vPriBakup);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_EccSign(uint hDongle, ushort wPriFileID, byte[] pHashData, uint nHashDataLen, byte[] pOutData);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_EccVerify(uint hDongle, ref ECCSM2_PUBLIC_KEY pPubKey, byte[] pHashData, uint nHashDataLen, byte[] pSign);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_SM2GenPubPriKey(uint hDongle, ushort wPriFileID, ref ECCSM2_PUBLIC_KEY pPubBakup, ref ECCSM2_PRIVATE_KEY pPriBakup);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_SM2Sign(uint hDongle, ushort wPriFileID, byte[] pHashData, uint nHashDataLen, byte[] pOutData);
        [DllImport("Dongle_d.dll")]
        static extern uint Dongle_SM2Verify(uint hDongle, ref ECCSM2_PUBLIC_KEY pPubKey, byte[] pHashData, uint nHashDataLen, byte[] pSign);


        private void test_Click(object sender, EventArgs e)
        {
            uint ret = 0;
            ushort pCount = 0;
            uint hDongle = 0;
            DONGLE_INFO pDongleInfo = new DONGLE_INFO();
            string str = "FFFFFFFFFFFFFFFF";//默认的开发商密码
            byte[] AdminPin = System.Text.Encoding.Default.GetBytes(str);
            int uiRemainCount = 0;
            short nInOutDataLen = 0;
            int   nMainRet   = 0;

            ret = Dongle_Enum(ref pDongleInfo, out pCount);
            if (ret != 0)
            {
                textBox1.Text = "Enum Dongle Failed! Return value:" + ret.ToString("X") + "\r\n";
                return;

            }
            else
            {
                textBox1.Text = "Enum Dongle Success!Count: " + pCount + "\r\n";
            }

            ret = Dongle_Open(ref hDongle, 0);
            if (ret != 0)
            {
                textBox1.Text += "Open Dongle Failed! Return value:" + ret.ToString("X") + "\r\n";
                return;

            }
            else
            {
                textBox1.Text += "Open Dongle Success! \r\n";
            }

            ret = Dongle_VerifyPIN(hDongle, 1, AdminPin, out uiRemainCount);
            if (ret != 0)
            {
                textBox1.Text += "Verify Dongle Failed! Return value:" + ret.ToString("X") + "\r\n";
                return;

            }
            else
            {
                textBox1.Text += "Verify Dongle Success! \r\n";
            }


            
            EXE_FILE_INFO []exeFileInfo=new EXE_FILE_INFO[2];

            //设置批量下载的可执行程序
            //设置第一个文件
            exeFileInfo[0].m_dwSize = EXE_SIZE_1;//文件大小
            exeFileInfo[0].m_Priv = 0;//匿名可调用
            exeFileInfo[0].m_wFileID = 0x1111;//文件ID
            exeFileInfo[0].m_pData = g_progExeFile1;

            //设置第二个文件
            exeFileInfo[1].m_dwSize = EXE_SIZE_2;//文件大小
            exeFileInfo[1].m_Priv = 0;//匿名可调用
            exeFileInfo[1].m_wFileID = 0x2222;//文件ID
            exeFileInfo[1].m_pData = g_progExeFile2;

            //批量下载可执行文件
            ret = Dongle_DownloadExeFile(hDongle, exeFileInfo, 2);
            if (ret != 0)
            {
                textBox1.Text += "Download Exe file Failed! Return value:" + ret.ToString("X") + "\r\n";
                return;

            }
            else
            {
                textBox1.Text += "Download Exe file Success! \r\n";
            }

            //运行可执行文件	
            nInOutDataLen = 20;
            byte []buffer=new byte[1024];
            for (int i = 0; i < buffer.Length; i++)
            {
                buffer[i] = 0;
            }
            ret = Dongle_RunExeFile(hDongle, 0x1111, buffer, nInOutDataLen, ref nMainRet);
            if (ret != 0)
            {
                textBox1.Text += "Run Exe file Failed! Return value:" + ret.ToString("X") + "\r\n";
                return;

            }
            else
            {
                textBox1.Text += "Run Exe file Success! \r\n";
            }
            
            //运行可执行文件
            nInOutDataLen = 32;
            ret = Dongle_RunExeFile(hDongle, 0x1111, buffer, nInOutDataLen, ref nMainRet);
            if (ret != 0)
            {
                textBox1.Text += "Run Exe file Failed! Return value:" + ret.ToString("X") + "\r\n";
                return;

            }
            else
            {
                textBox1.Text += "Run Exe file Success! \r\n";
            }
            
            ret = Dongle_Close(hDongle);
            if (ret != 0)
            {
                textBox1.Text += "Close Dongle Failed! Return value:" + ret.ToString("X") + "\r\n";
                return;

            }
            else
            {
                textBox1.Text += "Close Dongle Success! \r\n";
            }


            

        }
        public const ushort EXE_SIZE_1 = 1776;
        public const ushort EXE_SIZE_2 = 1808;

        public byte[] g_progExeFile1 = new byte[]{
            0x00,0x0c,0x00,0x68,0x55,0x00,0x00,0x00,0x59,0x00,0x00,0x00,0x5b,0x00,0x00,0x00,
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5d,0x00,0x00,0x00,
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5f,0x00,0x00,0x00,0x61,0x00,0x00,0x00,
            0x03,0x48,0x85,0x46,0x00,0xf0,0x22,0xfb,0x00,0x48,0x00,0x47,0xb1,0x03,0x00,0x00,
            0x00,0x0c,0x00,0x68,0x03,0x48,0x00,0x47,0xfe,0xe7,0xfe,0xe7,0xfe,0xe7,0xfe,0xe7,
            0xfe,0xe7,0x00,0x00,0x41,0x00,0x00,0x00,0x10,0xb5,0xd2,0x48,0x00,0xf0,0x00,0xfa,
            0xd0,0x48,0x10,0x21,0x00,0x1d,0x00,0xf0,0xf1,0xf9,0x10,0xbd,0x10,0xb5,0xcd,0x48,
            0x00,0xf0,0xda,0xf9,0x02,0x20,0x00,0xf0,0xe0,0xf9,0x10,0xbd,0x1c,0xb5,0x01,0x20,
            0x80,0x02,0x00,0x90,0x00,0x20,0x69,0x46,0x88,0x80,0xc8,0x80,0x08,0x23,0x6a,0x46,
            0xc5,0x49,0x01,0x20,0x00,0xf0,0x96,0xf9,0x1c,0xbd,0x10,0xb5,0xc2,0x49,0x01,0x20,
            0x00,0xf0,0xb8,0xf9,0x10,0xbd,0x38,0xb5,0x88,0x22,0x80,0x21,0xbd,0x48,0x00,0xf0,
            0xd1,0xfa,0xbc,0x4b,0xbd,0x4c,0x00,0x93,0x80,0x23,0x00,0x22,0x21,0x46,0x01,0x20,
            0x00,0xf0,0x9a,0xf9,0x09,0x21,0x09,0x03,0x88,0x42,0x09,0xd1,0x80,0x21,0xb5,0x48,
            0x00,0xf0,0xc7,0xfa,0xb3,0x4b,0x80,0x22,0x00,0x21,0x20,0x46,0x00,0xf0,0x7e,0xf9,
            0x38,0xbd,0x10,0xb5,0xaf,0x48,0x00,0xf0,0xdf,0xf9,0x20,0x21,0xad,0x48,0x00,0xf0,
            0xb8,0xfa,0xac,0x48,0x20,0x30,0x00,0xf0,0xce,0xf9,0x10,0xbd,0x10,0xb5,0x88,0xb0,
            0x10,0x20,0x05,0x90,0x00,0x20,0xaa,0x4c,0x06,0x90,0x08,0x23,0x05,0xaa,0x21,0x46,
            0x04,0x20,0x00,0xf0,0x57,0xf9,0x09,0x21,0x09,0x03,0x88,0x42,0x0b,0xd1,0x10,0x21,
            0x01,0xa8,0x00,0xf0,0x93,0xf9,0x01,0xab,0x00,0x93,0x10,0x23,0x00,0x22,0x21,0x46,
            0x04,0x20,0x00,0xf0,0x61,0xf9,0x08,0xb0,0x10,0xbd,0x70,0xb5,0x99,0x49,0x10,0x22,
            0x88,0x18,0x05,0x46,0x00,0xf0,0x6a,0xfa,0x99,0x4c,0x00,0x22,0x23,0x46,0x10,0x21,
            0x28,0x46,0x00,0xf0,0x28,0xfa,0x93,0x48,0x29,0x46,0x20,0x30,0x10,0x22,0x05,0x46,
            0x00,0xf0,0x5c,0xfa,0x23,0x46,0x01,0x22,0x10,0x21,0x28,0x46,0x00,0xf0,0x1b,0xfa,
            0x70,0xbd,0x70,0xb5,0x8b,0x49,0x10,0x22,0x88,0x18,0x05,0x46,0x00,0xf0,0x4e,0xfa,
            0x8b,0x4c,0x00,0x22,0x23,0x46,0x10,0x21,0x28,0x46,0x00,0xf0,0x18,0xfa,0x85,0x48,
            0x29,0x46,0x20,0x30,0x10,0x22,0x05,0x46,0x00,0xf0,0x40,0xfa,0x23,0x46,0x01,0x22,
            0x10,0x21,0x28,0x46,0x00,0xf0,0x0b,0xfa,0x70,0xbd,0x7e,0x4a,0x10,0xb5,0x20,0x32,
            0x20,0x21,0x50,0x1a,0x00,0xf0,0x0f,0xfa,0x10,0xbd,0x7a,0x4a,0x10,0xb5,0x20,0x32,
            0x20,0x21,0x50,0x1a,0x00,0xf0,0x11,0xfa,0x10,0xbd,0xf0,0xb5,0xb7,0xb0,0x03,0x20,
            0x69,0x46,0x88,0x84,0xff,0x20,0x01,0x30,0xc8,0x84,0x00,0x20,0xc0,0x43,0x74,0x4f,
            0x00,0x26,0x0a,0x90,0x0c,0x23,0x09,0xaa,0x39,0x46,0x0b,0x96,0x03,0x20,0x00,0xf0,
            0xe9,0xf8,0x09,0x25,0x2d,0x03,0x6b,0x4c,0xa8,0x42,0x03,0xd0,0x20,0x80,0xa7,0x80,
            0x37,0xb0,0xf0,0xbd,0x0c,0xa9,0x38,0x46,0x00,0xf0,0x83,0xf9,0xa8,0x42,0x02,0xd0,
            0x20,0x80,0x68,0x49,0x1a,0xe0,0x20,0x21,0x01,0xa8,0x00,0xf0,0x17,0xf9,0x68,0x46,
            0x06,0x71,0x36,0xab,0x00,0x93,0x26,0xab,0x20,0x22,0x01,0xa9,0x38,0x46,0x00,0xf0,
            0x7a,0xf9,0xa8,0x42,0x02,0xd0,0x20,0x80,0x5f,0x49,0x07,0xe0,0x26,0xab,0x20,0x22,
            0x01,0xa9,0x15,0xa8,0x00,0xf0,0x7e,0xf9,0x5c,0x49,0x20,0x80,0xa1,0x80,0xd7,0xe7,
            0xf0,0xb5,0xb7,0xb0,0x03,0x20,0x69,0x46,0x88,0x84,0x81,0x20,0x00,0x02,0xc8,0x84,
            0x00,0x20,0xc0,0x43,0x53,0x4f,0x00,0x26,0x0a,0x90,0x0c,0x23,0x09,0xaa,0x39,0x46,
            0x0b,0x96,0x03,0x20,0x00,0xf0,0xa6,0xf8,0x09,0x25,0x2d,0x03,0x49,0x4c,0xa8,0x42,
            0x02,0xd0,0x20,0x80,0x79,0x10,0x23,0xe0,0x0c,0xa9,0x38,0x46,0x00,0xf0,0x66,0xf9,
            0xa8,0x42,0x02,0xd0,0x20,0x80,0xa7,0x80,0xb2,0xe7,0x20,0x21,0x01,0xa8,0x00,0xf0,
            0xd5,0xf8,0x68,0x46,0x06,0x71,0x36,0xab,0x00,0x93,0x26,0xab,0x20,0x22,0x01,0xa9,
            0x38,0x46,0x00,0xf0,0x5d,0xf9,0xa8,0x42,0x02,0xd0,0x20,0x80,0x3e,0x49,0x07,0xe0,
            0x26,0xab,0x20,0x22,0x01,0xa9,0x15,0xa8,0x00,0xf0,0x61,0xf9,0x3b,0x49,0x20,0x80,
            0xa1,0x80,0x95,0xe7,0xf0,0xb5,0xff,0xb0,0x8c,0xb0,0x09,0xa8,0x00,0x90,0x08,0x90,
            0x37,0x48,0x06,0x90,0x07,0x46,0xff,0x30,0x21,0x30,0x07,0x90,0x02,0x20,0x69,0x46,
            0x08,0x81,0x80,0x02,0x48,0x81,0x00,0x20,0xc0,0x43,0x03,0x90,0x00,0x20,0x2e,0x4e,
            0x04,0x90,0x20,0x37,0x0c,0x23,0x02,0xaa,0x31,0x46,0x02,0x20,0x00,0xf0,0x5a,0xf8,
            0x09,0x25,0x2d,0x03,0x23,0x4c,0xa8,0x42,0x02,0xd0,0x20,0x80,0x24,0x49,0x07,0xe0,
            0x30,0x46,0x00,0x99,0x00,0xf0,0xc9,0xf8,0xa8,0x42,0x05,0xd0,0x21,0x49,0x20,0x80,
            0xa1,0x80,0x7f,0xb0,0x0c,0xb0,0xf0,0xbd,0x20,0x21,0x06,0x98,0x00,0xf0,0x86,0xf8,
            0x20,0x22,0x18,0x48,0x06,0x99,0x00,0xf0,0x69,0xf9,0x00,0x23,0x05,0xaa,0x01,0x93,
            0x00,0x92,0x3b,0x46,0x20,0x22,0x30,0x46,0x06,0x99,0x00,0xf0,0xb8,0xf8,0xa8,0x42,
            0x02,0xd0,0x20,0x80,0xa6,0x80,0xe4,0xe7,0x01,0x23,0x05,0xaa,0x01,0x93,0x00,0x92,
            0x68,0x46,0x81,0x8a,0x38,0x46,0x07,0x9b,0x08,0x9a,0x00,0xf0,0xb9,0xf8,0x04,0x46,
            0x08,0x48,0x20,0x22,0x20,0x30,0x07,0x99,0x00,0xf0,0x48,0xf9,0x05,0x49,0x0b,0x48,
            0x40,0x31,0x0c,0x80,0x88,0x80,0x68,0x46,0x80,0x8a,0x08,0x81,0x20,0x46,0xc8,0xe7,
            0x5a,0xe6,0x00,0x00,0x00,0x00,0x00,0x68,0x34,0x12,0x00,0x00,0xff,0xff,0x00,0x00,
            0x11,0x11,0x00,0x00,0x22,0x22,0x00,0x00,0x33,0x33,0x00,0x00,0x44,0x44,0x00,0x00,
            0x00,0x0c,0x00,0x68,0x70,0xb5,0x93,0x4c,0x10,0x25,0x25,0x80,0xa0,0x80,0xe1,0x80,
            0x91,0x48,0xe3,0x60,0xa2,0x60,0x80,0x47,0x60,0x88,0x70,0xbd,0x70,0xb5,0x8d,0x4c,
            0x11,0x25,0x25,0x80,0x01,0x25,0xa5,0x80,0xe0,0x80,0x21,0x81,0x62,0x81,0x8a,0x48,
            0xe3,0x60,0x80,0x47,0x60,0x88,0x70,0xbd,0x70,0xb5,0x86,0x4c,0x12,0x26,0x04,0x9d,
            0x26,0x80,0xa0,0x80,0xe1,0x80,0x22,0x81,0x63,0x81,0x83,0x48,0xe5,0x60,0x80,0x47,
            0x60,0x88,0x70,0xbd,0x10,0xb5,0x7f,0x4c,0x13,0x22,0x22,0x80,0xa0,0x80,0x7e,0x48,
            0xe1,0x80,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x7a,0x4c,0x14,0x21,0x21,0x80,
            0x60,0x60,0x79,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x75,0x4c,0x15,0x21,
            0x21,0x80,0x20,0x71,0x74,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x71,0x4c,
            0x16,0x22,0x22,0x80,0x60,0x60,0x70,0x48,0x21,0x72,0x80,0x47,0x60,0x88,0x10,0xbd,
            0x10,0xb5,0x6c,0x4c,0x17,0x21,0x21,0x80,0x60,0x60,0x6b,0x48,0x80,0x47,0x60,0x88,
            0x10,0xbd,0x10,0xb5,0x67,0x4c,0x18,0x21,0x21,0x80,0x60,0x60,0x66,0x48,0x80,0x47,
            0x60,0x88,0x10,0xbd,0x10,0xb5,0x63,0x4c,0x29,0x21,0x21,0x80,0x60,0x60,0x62,0x48,
            0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x5e,0x4c,0x19,0x21,0x21,0x80,0x60,0x60,
            0x5d,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x5a,0x4c,0x1a,0x21,0x21,0x80,
            0x60,0x60,0x59,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x55,0x4c,0x1b,0x22,
            0x22,0x80,0xa0,0x80,0x54,0x48,0xa1,0x60,0x80,0x47,0x60,0x88,0x10,0xbd,0xf8,0xb5,
            0x50,0x4c,0x1c,0x25,0x07,0x9f,0x06,0x9e,0x25,0x80,0xa3,0x60,0x61,0x60,0xa2,0x81,
            0xe0,0x81,0x4d,0x48,0x27,0x82,0x80,0x47,0xa0,0x89,0x30,0x80,0x60,0x88,0xf8,0xbd,
            0xf8,0xb5,0x48,0x4c,0x1d,0x25,0x07,0x9f,0x06,0x9e,0x25,0x80,0xa3,0x60,0x60,0x60,
            0x21,0x82,0xe2,0x60,0x44,0x48,0x67,0x82,0x80,0x47,0x20,0x8a,0x30,0x80,0x60,0x88,
            0xf8,0xbd,0x10,0xb5,0x3f,0x4c,0x1e,0x22,0x22,0x80,0xa0,0x80,0x3e,0x48,0xa1,0x60,
            0x80,0x47,0x60,0x88,0x10,0xbd,0x70,0xb5,0x3a,0x4c,0x1f,0x26,0x04,0x9d,0x26,0x80,
            0xa3,0x60,0x61,0x60,0xa2,0x81,0xe0,0x81,0x37,0x48,0x80,0x47,0xa0,0x89,0x28,0x80,
            0x60,0x88,0x70,0xbd,0x70,0xb5,0x33,0x4c,0x20,0x25,0x25,0x80,0xa1,0x60,0x60,0x60,
            0x22,0x82,0x31,0x48,0xe3,0x60,0x80,0x47,0x60,0x88,0x70,0xbd,0x10,0xb5,0x2d,0x4c,
            0x21,0x22,0x22,0x80,0xa0,0x80,0x2c,0x48,0xa1,0x60,0x80,0x47,0x60,0x88,0x10,0xbd,
            0x70,0xb5,0x28,0x4c,0x22,0x26,0x04,0x9d,0x26,0x80,0xa3,0x60,0x61,0x60,0xa2,0x81,
            0xe0,0x81,0x25,0x48,0x80,0x47,0xa0,0x89,0x28,0x80,0x60,0x88,0x70,0xbd,0x70,0xb5,
            0x20,0x4c,0x23,0x25,0x25,0x80,0xa1,0x60,0x60,0x60,0x22,0x82,0x1e,0x48,0xe3,0x60,
            0x80,0x47,0x60,0x88,0x70,0xbd,0x70,0xb5,0x1a,0x4c,0x24,0x25,0x25,0x80,0xa1,0x60,
            0x60,0x60,0xa2,0x81,0x18,0x48,0xe3,0x81,0x80,0x47,0x60,0x88,0x70,0xbd,0x70,0xb5,
            0x14,0x4c,0x25,0x25,0x25,0x80,0xa1,0x60,0x60,0x60,0xa2,0x81,0x12,0x48,0xe3,0x81,
            0x80,0x47,0x60,0x88,0x70,0xbd,0x10,0xb5,0x0e,0x4c,0x26,0x23,0x23,0x80,0x23,0x1d,
            0x07,0xc3,0x0d,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x09,0x4c,0x27,0x23,
            0x23,0x80,0x23,0x1d,0x07,0xc3,0x08,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,
            0x04,0x4c,0x28,0x23,0x23,0x80,0x23,0x1d,0x07,0xc3,0x03,0x48,0x80,0x47,0x60,0x88,
            0x10,0xbd,0x00,0x00,0x00,0x14,0x00,0x68,0x01,0x7f,0x00,0x00,0x70,0xb5,0x06,0x46,
            0x08,0x46,0x15,0x46,0x00,0xf0,0x0c,0xf8,0x34,0x46,0x03,0xe0,0x01,0x78,0x40,0x1c,
            0x21,0x70,0x64,0x1c,0x6d,0x1e,0xf9,0xd2,0x30,0x46,0x70,0xbd,0xee,0xe7,0xed,0xe7,
            0x0d,0x21,0xc9,0x06,0x88,0x42,0x02,0xd2,0x01,0x49,0xc9,0x6b,0x08,0x18,0x70,0x47,
            0xc0,0x1f,0x00,0x68,0xd2,0xb2,0x01,0xe0,0x02,0x70,0x40,0x1c,0x49,0x1e,0xfb,0xd2,
            0x70,0x47,0x00,0x22,0xf6,0xe7,0x10,0xb5,0x04,0x46,0x08,0x46,0x11,0x46,0x02,0x46,
            0x20,0x46,0xff,0xf7,0xef,0xff,0x20,0x46,0x10,0xbd,0x00,0x00,0x06,0x4c,0x01,0x25,
            0x06,0x4e,0x05,0xe0,0x20,0x46,0xe3,0x68,0x07,0xc8,0x2b,0x43,0x98,0x47,0x10,0x34,
            0xb4,0x42,0xf7,0xd3,0xff,0xf7,0xd0,0xfc,0xd0,0x06,0x00,0x00,0xf0,0x06,0x00,0x00,
            0x02,0xe0,0x08,0xc8,0x12,0x1f,0x08,0xc1,0x00,0x2a,0xfa,0xd1,0x70,0x47,0x70,0x47,
            0x00,0x20,0x01,0xe0,0x01,0xc1,0x12,0x1f,0x00,0x2a,0xfb,0xd1,0x70,0x47,0x00,0x00,
            0xf0,0x06,0x00,0x00,0x00,0x04,0x00,0x68,0x00,0x08,0x00,0x00,0xc0,0x06,0x00,0x00,
            0xf0,0x06,0x00,0x00,0x00,0x0c,0x00,0x68,0x00,0x04,0x00,0x00,0xc0,0x06,0x00,0x00};


        public byte[] g_progExeFile2 = new byte[]{
            0x00,0x0c,0x00,0x68,0x55,0x00,0x00,0x00,0x59,0x00,0x00,0x00,0x5b,0x00,0x00,0x00,
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5d,0x00,0x00,0x00,
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5f,0x00,0x00,0x00,0x61,0x00,0x00,0x00,
            0x03,0x48,0x85,0x46,0x00,0xf0,0x32,0xfb,0x00,0x48,0x00,0x47,0xd3,0x03,0x00,0x00,
            0x00,0x0c,0x00,0x68,0x03,0x48,0x00,0x47,0xfe,0xe7,0xfe,0xe7,0xfe,0xe7,0xfe,0xe7,
            0xfe,0xe7,0x00,0x00,0x41,0x00,0x00,0x00,0x10,0xb5,0xda,0x48,0x00,0xf0,0x10,0xfa,
            0xd8,0x48,0x10,0x21,0x00,0x1d,0x00,0xf0,0x01,0xfa,0x10,0xbd,0x10,0xb5,0xd5,0x48,
            0x00,0xf0,0xea,0xf9,0x02,0x20,0x00,0xf0,0xf0,0xf9,0x10,0xbd,0x1c,0xb5,0x01,0x20,
            0x80,0x02,0x00,0x90,0x00,0x20,0x69,0x46,0x88,0x80,0xc8,0x80,0x08,0x23,0x6a,0x46,
            0xcd,0x49,0x01,0x20,0x00,0xf0,0xa6,0xf9,0x1c,0xbd,0x10,0xb5,0xca,0x49,0x01,0x20,
            0x00,0xf0,0xc8,0xf9,0x10,0xbd,0x38,0xb5,0x88,0x22,0x80,0x21,0xc5,0x48,0x00,0xf0,
            0xe1,0xfa,0xc4,0x4b,0xc5,0x4c,0x00,0x93,0x80,0x23,0x00,0x22,0x21,0x46,0x01,0x20,
            0x00,0xf0,0xaa,0xf9,0x09,0x21,0x09,0x03,0x88,0x42,0x09,0xd1,0x80,0x21,0xbd,0x48,
            0x00,0xf0,0xd7,0xfa,0xbb,0x4b,0x80,0x22,0x00,0x21,0x20,0x46,0x00,0xf0,0x8e,0xf9,
            0x38,0xbd,0x10,0xb5,0xb7,0x48,0x00,0xf0,0xef,0xf9,0x20,0x21,0xb5,0x48,0x00,0xf0,
            0xc8,0xfa,0xb4,0x48,0x20,0x30,0x00,0xf0,0xde,0xf9,0x10,0xbd,0x10,0xb5,0x88,0xb0,
            0x10,0x20,0x05,0x90,0x00,0x20,0xb2,0x4c,0x06,0x90,0x08,0x23,0x05,0xaa,0x21,0x46,
            0x04,0x20,0x00,0xf0,0x67,0xf9,0x09,0x21,0x09,0x03,0x88,0x42,0x0b,0xd1,0x10,0x21,
            0x01,0xa8,0x00,0xf0,0xa3,0xf9,0x01,0xab,0x00,0x93,0x10,0x23,0x00,0x22,0x21,0x46,
            0x04,0x20,0x00,0xf0,0x71,0xf9,0x08,0xb0,0x10,0xbd,0x70,0xb5,0xa1,0x49,0x10,0x22,
            0x88,0x18,0x05,0x46,0x00,0xf0,0x7a,0xfa,0xa1,0x4c,0x00,0x22,0x23,0x46,0x10,0x21,
            0x28,0x46,0x00,0xf0,0x38,0xfa,0x9b,0x48,0x29,0x46,0x20,0x30,0x10,0x22,0x05,0x46,
            0x00,0xf0,0x6c,0xfa,0x23,0x46,0x01,0x22,0x10,0x21,0x28,0x46,0x00,0xf0,0x2b,0xfa,
            0x70,0xbd,0x70,0xb5,0x93,0x49,0x10,0x22,0x88,0x18,0x05,0x46,0x00,0xf0,0x5e,0xfa,
            0x93,0x4c,0x00,0x22,0x23,0x46,0x10,0x21,0x28,0x46,0x00,0xf0,0x28,0xfa,0x8d,0x48,
            0x29,0x46,0x20,0x30,0x10,0x22,0x05,0x46,0x00,0xf0,0x50,0xfa,0x23,0x46,0x01,0x22,
            0x10,0x21,0x28,0x46,0x00,0xf0,0x1b,0xfa,0x70,0xbd,0x10,0xb5,0x86,0xb0,0x14,0x21,
            0x68,0x46,0x00,0xf0,0x66,0xfa,0x6a,0x46,0x20,0x21,0x82,0x48,0x00,0xf0,0x1b,0xfa,
            0x04,0x46,0x01,0x21,0x89,0x02,0x7f,0x48,0x00,0xf0,0x5b,0xfa,0x14,0x22,0x69,0x46,
            0x7c,0x48,0x00,0xf0,0x33,0xfa,0x20,0x46,0x06,0xb0,0x10,0xbd,0x79,0x4a,0x10,0xb5,
            0x20,0x32,0x20,0x21,0x50,0x1a,0x00,0xf0,0x10,0xfa,0x10,0xbd,0xf0,0xb5,0xb7,0xb0,
            0x03,0x20,0x69,0x46,0x88,0x84,0xff,0x20,0x01,0x30,0xc8,0x84,0x00,0x20,0xc0,0x43,
            0x73,0x4f,0x00,0x26,0x0a,0x90,0x0c,0x23,0x09,0xaa,0x39,0x46,0x0b,0x96,0x03,0x20,
            0x00,0xf0,0xe8,0xf8,0x09,0x25,0x2d,0x03,0x6a,0x4c,0xa8,0x42,0x03,0xd0,0x20,0x80,
            0xa7,0x80,0x37,0xb0,0xf0,0xbd,0x0c,0xa9,0x38,0x46,0x00,0xf0,0x82,0xf9,0xa8,0x42,
            0x02,0xd0,0x20,0x80,0x67,0x49,0x1a,0xe0,0x20,0x21,0x01,0xa8,0x00,0xf0,0x16,0xf9,
            0x68,0x46,0x06,0x71,0x36,0xab,0x00,0x93,0x26,0xab,0x20,0x22,0x01,0xa9,0x38,0x46,
            0x00,0xf0,0x79,0xf9,0xa8,0x42,0x02,0xd0,0x20,0x80,0x5f,0x49,0x07,0xe0,0x26,0xab,
            0x20,0x22,0x01,0xa9,0x15,0xa8,0x00,0xf0,0x7d,0xf9,0x5c,0x49,0x20,0x80,0xa1,0x80,
            0xd7,0xe7,0xf0,0xb5,0xb7,0xb0,0x03,0x20,0x69,0x46,0x88,0x84,0x81,0x20,0x00,0x02,
            0xc8,0x84,0x00,0x20,0xc0,0x43,0x53,0x4f,0x00,0x26,0x0a,0x90,0x0c,0x23,0x09,0xaa,
            0x39,0x46,0x0b,0x96,0x03,0x20,0x00,0xf0,0xa5,0xf8,0x09,0x25,0x2d,0x03,0x49,0x4c,
            0xa8,0x42,0x02,0xd0,0x20,0x80,0x79,0x10,0x23,0xe0,0x0c,0xa9,0x38,0x46,0x00,0xf0,
            0x65,0xf9,0xa8,0x42,0x02,0xd0,0x20,0x80,0xa7,0x80,0xb2,0xe7,0x20,0x21,0x01,0xa8,
            0x00,0xf0,0xd4,0xf8,0x68,0x46,0x06,0x71,0x36,0xab,0x00,0x93,0x26,0xab,0x20,0x22,
            0x01,0xa9,0x38,0x46,0x00,0xf0,0x5c,0xf9,0xa8,0x42,0x02,0xd0,0x20,0x80,0x3e,0x49,
            0x07,0xe0,0x26,0xab,0x20,0x22,0x01,0xa9,0x15,0xa8,0x00,0xf0,0x60,0xf9,0x3b,0x49,
            0x20,0x80,0xa1,0x80,0x95,0xe7,0xf0,0xb5,0xff,0xb0,0x8c,0xb0,0x09,0xa8,0x00,0x90,
            0x08,0x90,0x37,0x48,0x06,0x90,0x07,0x46,0xff,0x30,0x21,0x30,0x07,0x90,0x02,0x20,
            0x69,0x46,0x08,0x81,0x80,0x02,0x48,0x81,0x00,0x20,0xc0,0x43,0x03,0x90,0x00,0x20,
            0x2d,0x4e,0x04,0x90,0x20,0x37,0x0c,0x23,0x02,0xaa,0x31,0x46,0x02,0x20,0x00,0xf0,
            0x59,0xf8,0x09,0x25,0x2d,0x03,0x23,0x4c,0xa8,0x42,0x02,0xd0,0x20,0x80,0x24,0x49,
            0x07,0xe0,0x30,0x46,0x00,0x99,0x00,0xf0,0xc8,0xf8,0xa8,0x42,0x05,0xd0,0x21,0x49,
            0x20,0x80,0xa1,0x80,0x7f,0xb0,0x0c,0xb0,0xf0,0xbd,0x20,0x21,0x06,0x98,0x00,0xf0,
            0x85,0xf8,0x20,0x22,0x17,0x48,0x06,0x99,0x00,0xf0,0x68,0xf9,0x00,0x23,0x05,0xaa,
            0x01,0x93,0x00,0x92,0x3b,0x46,0x20,0x22,0x30,0x46,0x06,0x99,0x00,0xf0,0xb7,0xf8,
            0xa8,0x42,0x02,0xd0,0x20,0x80,0xa6,0x80,0xe4,0xe7,0x01,0x23,0x05,0xaa,0x01,0x93,
            0x00,0x92,0x68,0x46,0x81,0x8a,0x38,0x46,0x07,0x9b,0x08,0x9a,0x00,0xf0,0xb8,0xf8,
            0x04,0x46,0x08,0x48,0x20,0x22,0x20,0x30,0x07,0x99,0x00,0xf0,0x47,0xf9,0x05,0x49,
            0x0a,0x48,0x40,0x31,0x0c,0x80,0x88,0x80,0x68,0x46,0x80,0x8a,0x08,0x81,0x20,0x46,
            0xc8,0xe7,0xf2,0xe6,0x00,0x00,0x00,0x68,0x34,0x12,0x00,0x00,0xff,0xff,0x00,0x00,
            0x11,0x11,0x00,0x00,0x22,0x22,0x00,0x00,0x33,0x33,0x00,0x00,0x44,0x44,0x00,0x00,
            0x00,0x0c,0x00,0x68,0x70,0xb5,0x93,0x4c,0x10,0x25,0x25,0x80,0xa0,0x80,0xe1,0x80,
            0x91,0x48,0xe3,0x60,0xa2,0x60,0x80,0x47,0x60,0x88,0x70,0xbd,0x70,0xb5,0x8d,0x4c,
            0x11,0x25,0x25,0x80,0x01,0x25,0xa5,0x80,0xe0,0x80,0x21,0x81,0x62,0x81,0x8a,0x48,
            0xe3,0x60,0x80,0x47,0x60,0x88,0x70,0xbd,0x70,0xb5,0x86,0x4c,0x12,0x26,0x04,0x9d,
            0x26,0x80,0xa0,0x80,0xe1,0x80,0x22,0x81,0x63,0x81,0x83,0x48,0xe5,0x60,0x80,0x47,
            0x60,0x88,0x70,0xbd,0x10,0xb5,0x7f,0x4c,0x13,0x22,0x22,0x80,0xa0,0x80,0x7e,0x48,
            0xe1,0x80,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x7a,0x4c,0x14,0x21,0x21,0x80,
            0x60,0x60,0x79,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x75,0x4c,0x15,0x21,
            0x21,0x80,0x20,0x71,0x74,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x71,0x4c,
            0x16,0x22,0x22,0x80,0x60,0x60,0x70,0x48,0x21,0x72,0x80,0x47,0x60,0x88,0x10,0xbd,
            0x10,0xb5,0x6c,0x4c,0x17,0x21,0x21,0x80,0x60,0x60,0x6b,0x48,0x80,0x47,0x60,0x88,
            0x10,0xbd,0x10,0xb5,0x67,0x4c,0x18,0x21,0x21,0x80,0x60,0x60,0x66,0x48,0x80,0x47,
            0x60,0x88,0x10,0xbd,0x10,0xb5,0x63,0x4c,0x29,0x21,0x21,0x80,0x60,0x60,0x62,0x48,
            0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x5e,0x4c,0x19,0x21,0x21,0x80,0x60,0x60,
            0x5d,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x5a,0x4c,0x1a,0x21,0x21,0x80,
            0x60,0x60,0x59,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x55,0x4c,0x1b,0x22,
            0x22,0x80,0xa0,0x80,0x54,0x48,0xa1,0x60,0x80,0x47,0x60,0x88,0x10,0xbd,0xf8,0xb5,
            0x50,0x4c,0x1c,0x25,0x07,0x9f,0x06,0x9e,0x25,0x80,0xa3,0x60,0x61,0x60,0xa2,0x81,
            0xe0,0x81,0x4d,0x48,0x27,0x82,0x80,0x47,0xa0,0x89,0x30,0x80,0x60,0x88,0xf8,0xbd,
            0xf8,0xb5,0x48,0x4c,0x1d,0x25,0x07,0x9f,0x06,0x9e,0x25,0x80,0xa3,0x60,0x60,0x60,
            0x21,0x82,0xe2,0x60,0x44,0x48,0x67,0x82,0x80,0x47,0x20,0x8a,0x30,0x80,0x60,0x88,
            0xf8,0xbd,0x10,0xb5,0x3f,0x4c,0x1e,0x22,0x22,0x80,0xa0,0x80,0x3e,0x48,0xa1,0x60,
            0x80,0x47,0x60,0x88,0x10,0xbd,0x70,0xb5,0x3a,0x4c,0x1f,0x26,0x04,0x9d,0x26,0x80,
            0xa3,0x60,0x61,0x60,0xa2,0x81,0xe0,0x81,0x37,0x48,0x80,0x47,0xa0,0x89,0x28,0x80,
            0x60,0x88,0x70,0xbd,0x70,0xb5,0x33,0x4c,0x20,0x25,0x25,0x80,0xa1,0x60,0x60,0x60,
            0x22,0x82,0x31,0x48,0xe3,0x60,0x80,0x47,0x60,0x88,0x70,0xbd,0x10,0xb5,0x2d,0x4c,
            0x21,0x22,0x22,0x80,0xa0,0x80,0x2c,0x48,0xa1,0x60,0x80,0x47,0x60,0x88,0x10,0xbd,
            0x70,0xb5,0x28,0x4c,0x22,0x26,0x04,0x9d,0x26,0x80,0xa3,0x60,0x61,0x60,0xa2,0x81,
            0xe0,0x81,0x25,0x48,0x80,0x47,0xa0,0x89,0x28,0x80,0x60,0x88,0x70,0xbd,0x70,0xb5,
            0x20,0x4c,0x23,0x25,0x25,0x80,0xa1,0x60,0x60,0x60,0x22,0x82,0x1e,0x48,0xe3,0x60,
            0x80,0x47,0x60,0x88,0x70,0xbd,0x70,0xb5,0x1a,0x4c,0x24,0x25,0x25,0x80,0xa1,0x60,
            0x60,0x60,0xa2,0x81,0x18,0x48,0xe3,0x81,0x80,0x47,0x60,0x88,0x70,0xbd,0x70,0xb5,
            0x14,0x4c,0x25,0x25,0x25,0x80,0xa1,0x60,0x60,0x60,0xa2,0x81,0x12,0x48,0xe3,0x81,
            0x80,0x47,0x60,0x88,0x70,0xbd,0x10,0xb5,0x0e,0x4c,0x26,0x23,0x23,0x80,0x23,0x1d,
            0x07,0xc3,0x0d,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,0x09,0x4c,0x27,0x23,
            0x23,0x80,0x23,0x1d,0x07,0xc3,0x08,0x48,0x80,0x47,0x60,0x88,0x10,0xbd,0x10,0xb5,
            0x04,0x4c,0x28,0x23,0x23,0x80,0x23,0x1d,0x07,0xc3,0x03,0x48,0x80,0x47,0x60,0x88,
            0x10,0xbd,0x00,0x00,0x00,0x14,0x00,0x68,0x01,0x7f,0x00,0x00,0x70,0xb5,0x06,0x46,
            0x08,0x46,0x15,0x46,0x00,0xf0,0x0c,0xf8,0x34,0x46,0x03,0xe0,0x01,0x78,0x40,0x1c,
            0x21,0x70,0x64,0x1c,0x6d,0x1e,0xf9,0xd2,0x30,0x46,0x70,0xbd,0xee,0xe7,0xed,0xe7,
            0x0d,0x21,0xc9,0x06,0x88,0x42,0x02,0xd2,0x01,0x49,0xc9,0x6b,0x08,0x18,0x70,0x47,
            0xc0,0x1f,0x00,0x68,0xd2,0xb2,0x01,0xe0,0x02,0x70,0x40,0x1c,0x49,0x1e,0xfb,0xd2,
            0x70,0x47,0x00,0x22,0xf6,0xe7,0x10,0xb5,0x04,0x46,0x08,0x46,0x11,0x46,0x02,0x46,
            0x20,0x46,0xff,0xf7,0xef,0xff,0x20,0x46,0x10,0xbd,0x00,0x00,0x06,0x4c,0x01,0x25,
            0x06,0x4e,0x05,0xe0,0x20,0x46,0xe3,0x68,0x07,0xc8,0x2b,0x43,0x98,0x47,0x10,0x34,
            0xb4,0x42,0xf7,0xd3,0xff,0xf7,0xc0,0xfc,0xf0,0x06,0x00,0x00,0x10,0x07,0x00,0x00,
            0x02,0xe0,0x08,0xc8,0x12,0x1f,0x08,0xc1,0x00,0x2a,0xfa,0xd1,0x70,0x47,0x70,0x47,
            0x00,0x20,0x01,0xe0,0x01,0xc1,0x12,0x1f,0x00,0x2a,0xfb,0xd1,0x70,0x47,0x00,0x00,
            0x10,0x07,0x00,0x00,0x00,0x04,0x00,0x68,0x00,0x08,0x00,0x00,0xe0,0x06,0x00,0x00,
            0x10,0x07,0x00,0x00,0x00,0x0c,0x00,0x68,0x00,0x04,0x00,0x00,0xe0,0x06,0x00,0x00};

    }
}